using System;
using System.Globalization;
using UnityEngine;

namespace DoesNotCat {
    public class RealFakeLabias : MVRScript {

        private const string pluginName = "RealFakeLabias";
        private readonly string[] morphNames = {
            "Labia majora-spread-LLow",
            "Labia majora-spread-LMid",
            "Labia majora-spread-LUp",
            "Labia majora-spread-RLow",
            "Labia majora-spread-RMid",
            "Labia majora-spread-RUp",
            "Labia minora-spread-LLow",
            "Labia minora-spread-LMid",
            "Labia minora-spread-LUp",
            "Labia minora-spread-RLow",
            "Labia minora-spread-RMid",
            "Labia minora-spread-RUp"
        };

        private Atom atom;
        private DAZMorph[] morphs;
        private DAZBone lThigh, rThigh, pelvis;
        public JSONStorableFloat spreading, spreadingOuter, spreadingInner, spreadingFront, spreadingMid, spreadingBack, bias, biasInner, biasOuter, biasFront, biasMid, biasBack;

        public override void Init () {
            atom = GetContainingAtom ();
            if (atom.type != "Person") {
                LogError ($"Plugin only works on atoms with type 'Person', but found an atom of type '{containingAtom.type}'. Plugin will not work on this atom.");
                enabledJSON.SetVal (false);
                return;
            }
            JSONStorable geo = atom.GetStorableByID ("geometry");
            DAZCharacterSelector character = geo as DAZCharacterSelector;
            DAZBones bones = character.rootBones;
            lThigh = bones.GetDAZBone ("lThigh");
            rThigh = bones.GetDAZBone ("rThigh");
            pelvis = bones.GetDAZBone ("pelvis");
            if (lThigh == null || rThigh == null || pelvis == null) {
                LogError ($"One of the necessary bones 'lThight', 'rThight' or 'pelvis' could not be found on this atom. Plugin will not work on this atom.");
                enabledJSON.SetVal (false);
                return;
            }

            morphs = new DAZMorph[morphNames.Length];
            GenerateDAZMorphsControlUI mc = character.morphsControlUI;
            for (byte i = 0; i < morphNames.Length; ++i) {
                morphs[i] = mc.GetMorphByDisplayName (morphNames[i]);
                if (morphs[i] == null) {
                    LogError ($"The necessary morph '{morphNames[i]}' could not be found on this atom. Plugin will not work on this atom.");
                    enabledJSON.SetVal (false);
                    return;
                }
                if (morphs[i].isDriven && morphs[i].drivenBy != pluginName) {
                    LogError ($"The necessary morph '{morphNames[i]}' is already driven by '{morphs[i].drivenBy}'. Plugin will not work on this atom.");
                    enabledJSON.SetVal (false);
                    return;
                }
            }

            // Parameters
            spreading = new JSONStorableFloat ("Spreading factor", 1f, 0f, 2f, false, true);
            spreadingOuter = new JSONStorableFloat ("Spreading factor outer labia", 0.7f, 0f, 2f, false, true);
            spreadingInner = new JSONStorableFloat ("Spreading factor inner labia", 0.3f, 0f, 2f, false, true);
            spreadingFront = new JSONStorableFloat ("Spreading factor front", 1f, 0f, 2f, false, true);
            spreadingMid = new JSONStorableFloat ("Spreading factor mid", 1f, 0f, 2f, false, true);
            spreadingBack = new JSONStorableFloat ("Spreading factor back", 1f, 0f, 2f, false, true);
            bias = new JSONStorableFloat ("Bias", 0f, -1f, 1f, false, true);
            biasOuter = new JSONStorableFloat ("Bias outer labia", 0f, -1f, 1f, false, true);
            biasInner = new JSONStorableFloat ("Bias inner labia", 0f, -1f, 1f, false, true);
            biasFront = new JSONStorableFloat ("Bias front", 0f, -1f, 1f, false, true);
            biasMid = new JSONStorableFloat ("Bias mid", 0f, -1f, 1f, false, true);
            biasBack = new JSONStorableFloat ("Bias back", 0f, -1f, 1f, false, true);
            RegisterFloat (spreading);
            RegisterFloat (spreadingOuter);
            RegisterFloat (spreadingInner);
            RegisterFloat (spreadingFront);
            RegisterFloat (spreadingMid);
            RegisterFloat (spreadingBack);
            RegisterFloat (bias);
            RegisterFloat (biasInner);
            RegisterFloat (biasOuter);
            RegisterFloat (biasFront);
            RegisterFloat (biasMid);
            RegisterFloat (biasBack);
            CreateSlider (spreading);
            CreateSlider (spreadingOuter);
            CreateSlider (spreadingInner);
            CreateSlider (spreadingFront);
            CreateSlider (spreadingMid);
            CreateSlider (spreadingBack);
            CreateSlider (bias, true);
            CreateSlider (biasInner, true);
            CreateSlider (biasOuter, true);
            CreateSlider (biasFront, true);
            CreateSlider (biasMid, true);
            CreateSlider (biasBack, true);
        }

        public void Start () {
            foreach (DAZMorph morph in morphs) {
                morph.SetDriven (true, pluginName, true);
            }
        }

        public void Update () {
            Vector3 left = lThigh.transform.up;
            Vector3 right = rThigh.transform.up;
            Vector3 forward = pelvis.transform.forward;
            Vector3 sideward = pelvis.transform.right;
            float lForward = Vector3.Dot (left, forward);
            float lSideward = Vector3.Dot (left, sideward);
            float rForward = Vector3.Dot (right, forward);
            float rSideward = -Vector3.Dot (right, sideward);
            float lLow = (Math.Abs (-1 - lForward) / 2) * lSideward * spreadingBack.val + bias.val + biasBack.val;
            float lMid = lSideward * spreadingMid.val + bias.val + biasMid.val;
            float lUp = (Math.Abs (1 - lForward) / 2) * lSideward * spreadingFront.val + bias.val + biasFront.val;
            float rLow = (Math.Abs (-1 - rForward) / 2) * rSideward * spreadingBack.val + bias.val + biasBack.val;
            float rMid = rSideward * spreadingMid.val + bias.val + biasMid.val;
            float rUp = (Math.Abs (1 - rForward) / 2) * rSideward * spreadingFront.val + bias.val + biasFront.val;
            morphs[0].SetValue (lLow * spreading.val * spreadingOuter.val + biasOuter.val);
            morphs[1].SetValue (lMid * spreading.val * spreadingOuter.val + biasOuter.val);
            morphs[2].SetValue (lUp * spreading.val * spreadingOuter.val + biasOuter.val);
            morphs[3].SetValue (rLow * spreading.val * spreadingOuter.val + biasOuter.val);
            morphs[4].SetValue (rMid * spreading.val * spreadingOuter.val + biasOuter.val);
            morphs[5].SetValue (rUp * spreading.val * spreadingOuter.val + biasOuter.val);
            morphs[6].SetValue (lLow * spreading.val * spreadingInner.val + biasInner.val);
            morphs[7].SetValue (lMid * spreading.val * spreadingInner.val + biasInner.val);
            morphs[8].SetValue (lUp * spreading.val * spreadingInner.val + biasInner.val);
            morphs[9].SetValue (rLow * spreading.val * spreadingInner.val + biasInner.val);
            morphs[10].SetValue (rMid * spreading.val * spreadingInner.val + biasInner.val);
            morphs[11].SetValue (rUp * spreading.val * spreadingInner.val + biasInner.val);
        }

        void OnDestroy () {
            foreach (DAZMorph morph in morphs) {
                if (morph.drivenBy == pluginName) {
                    morph.SetDriven (true, pluginName, true);
                }
            }
        }

        void LogError (string error) {
            SuperController.LogError ($"[{pluginName}] {error}");
        }
    }
}